@@ -407,23 +407,48 @@ class MD_DataIdentification(object):
 
             self.uricode = []
             _values = md.findall(util.nspath_eval(
-                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:code/gco:CharacterString',
+                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:MD_Identifier/gmd:code/gco:CharacterString',
                 namespaces))
             _values += md.findall(util.nspath_eval(
-                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:MD_Identifier/gmd:code/gco:CharacterString',
+                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:code/gco:CharacterString',
                 namespaces))
             for i in _values:
                 val = util.testXMLValue(i)
-                if val is not None:
+                if val not in [None,'']:
                     self.uricode.append(val)
+            
+            _values = md.findall(util.nspath_eval(
+                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:MD_Identifier/gmd:code/gmx:Anchor',
+                namespaces))
+            _values += md.findall(util.nspath_eval(
+                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:code/gmx:Anchor',
+                namespaces))
+            for i in _values:
+                val = util.testXMLValue(i)
+                val1 = i.attrib.get(util.nspath_eval('xlink:href', namespaces))
+                if val1 not in [None,'']:
+                    self.uricode.append(val1)
+                elif val not in [None,'']:
+                    self.uricode.append(val)
+                    
 
             self.uricodespace = []
             for i in md.findall(util.nspath_eval(
                     'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:codeSpace/gco:CharacterString',
                     namespaces)):
                 val = util.testXMLValue(i)
-                if val is not None:
+                if val not in [None,'']:
                     self.uricodespace.append(val)
+            for i in md.findall(util.nspath_eval(
+                    'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:codeSpace/gmx:Anchor',
+                    namespaces)):
+                val = util.testXMLValue(i)
+                val1 = i.attrib.get(util.nspath_eval('xlink:href', namespaces))
+                if val1 not in [None,'']:
+                    self.uricode.append(val1)
+                elif val not in [None,'']:
+                    self.uricode.append(val)
+
 
             self.date = []
             self.datetype = []
@@ -477,6 +502,15 @@ class MD_DataIdentification(object):
                 val = util.testXMLValue(i)
                 if val is not None:
                     self.otherconstraints.append(val)
+            for i in md.findall(util.nspath_eval(
+                    'gmd:resourceConstraints/gmd:MD_LegalConstraints/gmd:otherConstraints/gmx:Anchor',
+                    namespaces)):
+                val = util.testXMLAttribute(i, util.nspath('href', namespaces["xlink"]))
+                if val is None:
+                    val = util.testXMLValue(i)
+                if val is not None:
+                    self.otherconstraints.append(val)    
+                    
 
             self.securityconstraints = []
             for i in md.findall(util.nspath_eval(
